#!/usr/bin/env/pyhton3

"""
MARLIN IDent
c. Rahul Tandon, R.S. Aqua, 2024
E: r.tandon@rsaqua.co.uk

Optimised IDent data srtucture for optimised data statistics and data reporting.
"No python" approach in order to compile to machine code with native datatypes. 


"""

# Standard imports
import numpy as np
import time as t

# Optmisied imports
from numba import int32, float32
from numba.experimental import jitclass


IdentDataClassSpec = [      # a simple scalar field
    ('data_source', float32[:]),         # an array field
]

@jitclass(IdentDataClassSpec)
class IdentData(object):
    
    def __init__(self, data_source : np.ndarray = None):
        self.data_source = data_source
    
    def fft_it(self):
        n = len(self.data_source)
        Y = np.fft.fft(self.data_source)/n
        Y = Y[:n//2]
        return Y
        # amplitudes =  abs(Y) 
        # max_power = np.max(amplitudes)
        
    def estimate_f_p_domain(self, Y):
        amplitudes = np.abs(Y)
        return amplitudes
        

if __name__ == "__main__":
    import matplotlib.pyplot as plt
    
    print ("Debug Tester")
    print ("################")
    
    source_data = None
    #load source data
   
    sr = 576000
    source_filepath = "/home/vixen/rs/dev/marlin_hp/marlin_hp/data/sim/streamedfile_999876447769858733241497.dat"
    with open(source_filepath, 'rb') as fr:
        c = fr.read()
        np_data = None
        dtype = np.dtype("float32")
        source_data  = np.frombuffer(c, dtype=dtype)
    
    # print (source_data)
    
    


    tester = IdentData(source_data)

    print ('Building fft - np.nfft')
    fft_start = t.time()

    Y = tester.fft_it()
    
    fff_end = t.time()
    fft_run_time = fff_end - fft_start
    print (f'FFT took {fft_run_time} (s)')
    print ("################")
    
    print ('Building fft - np.nfft')
    fft_start = t.time()

    Y = tester.fft_it()
    
    fff_end = t.time()
    fft_run_time = fff_end - fft_start
    print (f'FFT took {fft_run_time} (s)')
    print ("################")
    
    # print (Y)

    print ('Building Power')
    power_start = t.time()

    amplitudes = tester.estimate_f_p_domain(Y)
    
    power_end = t.time()
    power_run_time = power_end - power_start
    print (f'Power took {power_run_time} (s)')
    print ("################")
    
    print ('Building Power')
    power_start = t.time()

    amplitudes = tester.estimate_f_p_domain(Y)
    
    power_end = t.time()
    power_run_time = power_end - power_start
    print (f'Power took {power_run_time} (s)')
    print ("################")
    
    print (amplitudes.shape)
    n = len(source_data)
    k = np.arange(n)
    print (k)
    T = n/sr
    
    frq = k/T # two sides frequency range
    frq = frq[:len(frq)//2] # one side frequency range
    print (len(frq))
    
    plt.plot(frq,amplitudes,color='green',linewidth=0.1, markersize=0.5) 
    plt.title(f'Power Spectrum')
    plt.xlabel('f (Hz)')
    plt.ylabel('|P(freq)|')

    plt.xlim(0, 200)
    plt.savefig(f'powerprofile_.png')
    plt.close()
        
    
   
    


